// constants
const MONTHLY_LABEL = "      -";
const COMPLETED_LABEL = "      |";
const OCCURRENCE_ONCE_LABEL = "     x";
const SHOW_IN_TOOLTIP_LABEL = "          x";

var origReminders = null;
var origTodos = null;

var	selectCalendarSync = false;
var calendarReminderArray = new Array(12);

var lastSearchString = null;

function populateDateList() {
	var currentDate = oeDatePicker.gSelectedDate;
	if ( currentDate == null ) {
		currentDate = new Date();
	}

	var year = parseInt(currentDate.getFullYear());

	var dayList = document.getElementById('daylist');
	items = dayList.firstChild.childNodes;
	// remove ending nodes that could possibly be different each month
	// (i.e. any date greater than 28)
	var length = dayList.firstChild.childNodes.length - 1;	
	for ( var j = length; j >= 28; j-- ) {
		dayList.firstChild.removeChild( items[j] );
	}
	
	
	// now add proper # of days for this month
	var dayArray;
	if (reminderFox_isLeapYear(parseInt(year))) {
		dayArray = REMINDER_FOX_lDOMonth;
	}
	else {
		dayArray = REMINDER_FOX_DOMonth;
	}

	var monthlist = document.getElementById('monthlist');
	var currentMonth = reminderFox_getMonthAsInteger(monthlist.label);
	// if it's the Monthly special-case, treat it as a full month (31 days)
	if ( currentMonth == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
		currentMonth = 0;  
	}

	var numOfDays = dayArray[currentMonth];
 	for ( var k = 29; k < numOfDays + 1; k++ ) {
		var newDay = document.createElement("menuitem");
		var str = k + "";
		newDay.setAttribute("label", str);
		dayList.firstChild.appendChild(newDay);
	}
	

	// now make sure the dateList is not showing a date greater than
	// the number of days in the selected month; if it is set the list
	// to show the last day of the month
	var dayname = parseInt(dayList.label);

	var numOfDaysInt = parseInt( numOfDays );
	if ( dayname > numOfDaysInt ) {
		dayList.selectedItem = dayList.firstChild.childNodes[numOfDaysInt - 1];
	}
}


function populateDateListAndSelectNewReminderText() { 
	populateDateList();
	selectNewReminderText();
}

function selectTab() {
	// give focus to the reminder/todo description when switching tabs
	var remindersTab = document.getElementById("remindersTab");
	if ( remindersTab.selected ) {
		selectNewReminderText();
	}
	else {
		var todosTab = document.getElementById("todosTab");
		if ( todosTab.selected ) {
			var desc = document.getElementById("newTodoText");
	    	desc.focus();
	    }
	}
	
}
	
function loadReminders() {
	origReminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	origTodos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);	

	fillList(true, true);
	
	// select appropriate tab
	var currentTab = window.arguments[0];
	if ( currentTab == "todos" ) {
		var todosTab = document.getElementById("todosTab");
		var tabbox = document.getElementById("tabbox");
		tabbox.selectedTab = todosTab;		
	}
	
   	var todaysDate = new Date();
	oeDatePicker.onpopupshowing(todaysDate);
	selectCalendarSync = true;
	
	// show calendar based on button state
	var calendar = document.getElementById("oe-date-picker-overlay-box");
	var calendarToggleButton= document.getElementById("calendarToggle");
	if ( calendarToggleButton.checked ) {
		calendarToggleButton.setAttribute("tooltiptext", "Hide Calendar");	
		calendar.setAttribute("hidden", "false");	
	}
	else {
		calendarToggleButton.setAttribute("tooltiptext", "Show Calendar");	
		calendar.setAttribute("hidden", "true");	
	}

	
}

function fillList(  fillReminders,  fillTodoList) {
	if ( fillReminders ) {
		var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
		
		if (reminders != "") {	
			var remindersArray = reminders.split(REMINDER_SEPARATOR);
			for(i = 0; i < remindersArray.length; i++) {
				var reminder = remindersArray[i];
				var reminderArray = reminder.split(REMINDER_PROPERTY_DELIMITER);
				createListItem(reminderArray, false);
			}
		}
	}
	
	if ( fillTodoList ) {
		var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
		
		if (todos != "") {	
			var todosArray = todos.split(REMINDER_SEPARATOR);
			for(i = 0; i < todosArray.length; i++) {
				var todo = todosArray[i];
				var todoArray = todo.split(REMINDER_PROPERTY_DELIMITER);
				createTodoListItem(todoArray, false);
			}
		}
	}
	
	
	// now that we've added the sorted reminders in the list, let's
	// select (highlight) today's reminder or the next upcoming reminder
	highlightClosestUpcomingReminder();
	
	highlightTodo();
	
	// set the current date for adding new reminders
	var currentDate = new Date();
	setNewReminderDate(currentDate, false);

   	selectNewReminderText();
}

function doFindNext() {
	// only bring up find dialog if reminders tab is selected
	var remindersTab = document.getElementById("remindersTab");
	if ( !remindersTab.selected ) {
		return;
	}
	
	if ( lastSearchString == null ) {
		doFind();
	}
	else {
		findReminder();
	}
}

function doFind() {
	// only bring up find dialog if reminders tab is selected
	var remindersTab = document.getElementById("remindersTab");
	if ( !remindersTab.selected ) {
		return;
	}

	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	var result = { value: lastSearchString };
	var dummy = { value: 0 };
	if (gPromptService.prompt(window,
                            "Find Reminder",
                            "Enter text to search for:",
                            result,
                            null,
                            dummy)) {
                            
            lastSearchString = result.value;
          
        	findReminder();             
    }    
}


function findReminder( ) {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	// if nothing selected, default to start of list
	if ( selectedTreeItemIndex == -1 ) {
		selectedTreeItemIndex = 0;
	}
		
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];

	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
   caseInsensitiveLastSearchString = lastSearchString.toUpperCase();
	var treeChildren = document.getElementById("treechildren");
	var treeitems = treeChildren.childNodes;
	var startIndex = selectedTreeItemIndex;
	if ( startIndex + 1 < treeitems.length ) {
		startIndex = startIndex + 1;
	}

	var foundIndex = -1;
	for (var i=startIndex; i < treeitems.length; i++) {
		var item = treeitems[i];
		var row = item.childNodes[0];
		var descCell = row.childNodes[1];
		var descCellLabel = descCell.getAttribute("label");
		descCellLabel = descCellLabel.toUpperCase();

		if ( descCellLabel.indexOf( caseInsensitiveLastSearchString ) != -1  ) {
			foundIndex = i;

			break;
		}
	}
	
	if ( foundIndex == -1 ) {
		for (var i=0; i < startIndex; i++) {
			var item = treeitems[i];
			var row = item.childNodes[0];
			var descCell = row.childNodes[1];
			var descCellLabel = descCell.getAttribute("label");
			descCellLabel = descCellLabel.toUpperCase();

			if ( descCellLabel.indexOf( caseInsensitiveLastSearchString ) != -1  ) {
				foundIndex = i;
				break;
			}
		}
	}
	
	// if we found a search match, then highlight the new reminder in the list
	if ( foundIndex != -1 ) {		
		var treeSelection = document.getElementById("reminderTree").view.selection		
		treeSelection.select(foundIndex);
		
		// make sure that newly selected row is shown in scroll pane
		var value = document.getElementById("treeparent").value;
	  	var tree = document.getElementById("reminderTree");			
		var boxobject = tree.boxObject;
		boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
		
		// make sure it shows up at the top if possible
		var first = boxobject.getFirstVisibleRow();
		var last = boxobject.getLastVisibleRow();
		if ( foundIndex < first || foundIndex > last ) {				
			boxobject.scrollToRow(treeitems.length - 9);
		}
		
		var previousReminderOffset = 3;
		if ( foundIndex < previousReminderOffset ) { 
			boxobject.ensureRowIsVisible(i);
		}
		else {
			boxobject.ensureRowIsVisible(foundIndex  - previousReminderOffset);
		}
	}
}


function selectNewReminderText() {
    var desc = document.getElementById("newReminderText");  
    desc.focus();
}

function setNewReminderDate( date, repopulateDays ) {
	var month = date.getMonth();
	var day = date.getDate() - 1; // account for 1-offset
	

	var monthlist = document.getElementById('monthlist');
	var prefMonth = reminderFox_getMonthAsInteger(monthlist.label);

	// if the month has changed, we need to repopulate the day drop-down list
	if ( prefMonth != month ) {
		repopulateDays = true;
	}	
	
	var items = monthlist.firstChild.childNodes;
    monthlist.selectedItem = items[month];
    
    if ( repopulateDays ) {
   		populateDateList();
   	}
    
	var dayList = document.getElementById('daylist');
	items = dayList.firstChild.childNodes;
    dayList.selectedItem = items[day];	
}

function createMonthlyReminderListItems(valueArray) {
	for ( var i = 0; i < 12; i++ ) {
		valueArray[0] = i + "";
		createListItem( valueArray, true );
	}
}

function toggleCalendar() {
	var calendar = document.getElementById("oe-date-picker-overlay-box");
	var calendarToggleButton= document.getElementById("calendarToggle");
	if ( calendarToggleButton.checked ) {
		calendar.setAttribute("hidden", "true");	
		calendarToggleButton.setAttribute("tooltiptext", "Show Calendar");	
	}
	else {
		calendar.setAttribute("hidden", "false");	
		calendarToggleButton.setAttribute("tooltiptext", "Hide Calendar");	
	}
}

function reminderSelected(event) {
	if ( selectCalendarSync ) {
		var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
		if ( selectedTreeItemIndex >= 0 ) {
			var treeChildren = document.getElementById("treechildren");
			var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
			var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
			var dateLabelArray = dateLabel.split(" ");
			var monthAsInteger = reminderFox_getMonthAsInteger(dateLabelArray[0]);		
			var day = dateLabelArray[1];
			var year = oeDatePicker.gSelectedDate.getFullYear();
			if ( year == null ) {
				year = new Date().getFullYear();
			}
			var newDate = new Date(year, monthAsInteger, day );	
			oeDatePicker.onpopupshowing(newDate);
		}
	}
}


function createCalendarReminderArray() {
	calendarReminderArray = new Array(12);
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	if (reminders != "") {	
		var remindersArray = reminders.split(REMINDER_SEPARATOR);
		for(i = 0; i < remindersArray.length; i++) {
			var reminder = remindersArray[i];
			var valueArray = reminder.split(REMINDER_PROPERTY_DELIMITER);
						
			if ( valueArray[0] == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
					for ( var k= 0; k < 12; k++ ) {
						monthReplacement= k + "";
						
						var monthArray = calendarReminderArray[monthReplacement];
						if ( monthArray == null ) {
							monthArray = new Array(31);
							calendarReminderArray[monthReplacement] = monthArray;
						}
						
						var monthDay = parseInt((valueArray[1]));
						monthDay = reminderFox_getValidDateForMonth( new Date().getFullYear(), valueArray[0], monthDay );
					
						var dayReminderArray = monthArray[monthDay - 1];
						if ( dayReminderArray == null ) {
							dayReminderArray = new Array();
							monthArray[monthDay - 1] = dayReminderArray;
						}
						var newDayReminder = new ReminderContainer( valueArray[2], false );
									var  length = dayReminderArray.length;
						
						dayReminderArray[length] = newDayReminder;
		
						// check for any options that are specified, and mark them in the columns 
						if ( valueArray.length >= 4 ) {
				            for ( j = 3; j < valueArray.length; j++ ) {
				                var val = valueArray[j];
								 if ( val == REMINDER_IMPORTANT_FLAG ) {
				                    newDayReminder.important = true;
				                    break;
				                }				
				            }
				        }

					}								
				}
		else {		
		var monthArray = calendarReminderArray[valueArray[0]];
		if ( monthArray == null ) {
			monthArray = new Array(31);
			calendarReminderArray[valueArray[0]] = monthArray;
		}

		
		var monthDay = parseInt((valueArray[1]));
		var dayReminderArray = monthArray[monthDay - 1];
		if ( dayReminderArray == null ) {
			dayReminderArray = new Array();
			monthArray[monthDay - 1] = dayReminderArray;
		}
		var newDayReminder = new ReminderContainer( valueArray[2], false );
					var  length = dayReminderArray.length;
		dayReminderArray[length] = newDayReminder;
		
		// check for any options that are specified, and mark them in the columns 
		if ( valueArray.length >= 4 ) {
            for ( j = 3; j < valueArray.length; j++ ) {
                var val = valueArray[j];
				 if ( val == REMINDER_IMPORTANT_FLAG ) {
                    newDayReminder.important = true;
                    break;
                }

            }
        }
		}
	}
	}
}



function createListItem(valueArray, isMonthly) {
	var once = document.getElementById("occurrence");
	var treeChildren = document.getElementById("treechildren");
	var newItem = document.createElement("treeitem");
	var newRow = document.createElement("treerow");
	var newDateLabel = document.createElement("treecell");
	var newDescLabel = document.createElement("treecell");
	
	newItem.appendChild(newRow);
	newRow.appendChild(newDateLabel);
	newRow.appendChild(newDescLabel);
	var isImportant = false;
	// if it's a monthly reminder, add it for each month
	if ( valueArray[0] == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
		createMonthlyReminderListItems(valueArray);
	}
	else {		
		var dayVal = valueArray[1];
		var monthArray = calendarReminderArray[valueArray[0]];
		if ( monthArray == null ) {
			monthArray = new Array(31);
			calendarReminderArray[valueArray[0]] = monthArray;
		}


		if ( isMonthly ) {
			dayVal = reminderFox_getValidDateForMonth( new Date().getFullYear(), valueArray[0], dayVal );
		}
		var monthDay = parseInt(dayVal);
	
		var dayReminderArray = monthArray[monthDay - 1];
		if ( dayReminderArray == null ) {
			dayReminderArray = new Array();
			monthArray[monthDay - 1] = dayReminderArray;

		}
		var newDayReminder = new ReminderContainer( valueArray[2], false );
		var  length = dayReminderArray.length;
		
		dayReminderArray[length] = newDayReminder;

		var monthAsText = reminderFox_getMonthAsText(valueArray[0]);		
		newDateLabel.setAttribute("label", monthAsText + " " + dayVal);
		newDescLabel.setAttribute("label", valueArray[2]);
		
		var newOccurrenceLabel = document.createElement("treecell");
		newRow.appendChild(newOccurrenceLabel);
		
		var monthlyLabel = document.createElement("treecell");
		newRow.appendChild(monthlyLabel);		
		
		var newCompleteLabel = document.createElement("treecell");
		newRow.appendChild(newCompleteLabel);
		
		var newRemindUntilCompletedLabel = document.createElement("treecell");
		newRow.appendChild(newRemindUntilCompletedLabel);
		
		// check for any options that are specified, and mark them in the columns 
		if ( valueArray.length >= 4 ) {
            for ( j = 3; j < valueArray.length; j++ ) {
                var val = valueArray[j];
                if ( val == REMINDER_FOX_OCCURRENCE_ONCE ) {
                    newOccurrenceLabel.setAttribute("label", OCCURRENCE_ONCE_LABEL);			
                }
                else if ( val == REMINDER_IMPORTANT_FLAG ) {
                    // mark the date and description as important (red-highlighted)
                    newDateLabel.setAttribute("properties", "important");
                    newDescLabel.setAttribute("properties", "important");
                    isImportant = true;
                }
                else if ( val == REMINDER_TO_BE_COMPLETED_DELIMITER ) {
                	newCompleteLabel.setAttribute("label", COMPLETED_LABEL);		
                }
                else if ( val.indexOf( REMINDER_TO_BE_COMPLETED_DELIMITER ) == 0 && val != REMINDER_TO_BE_COMPLETED_DELIMITER) {
                    newRemindUntilCompletedLabel.setAttribute("label", COMPLETED_LABEL);			
                }
            }
        }
		if ( isMonthly ) {
			monthlyLabel.setAttribute("label", MONTHLY_LABEL);
		}

		if ( isImportant ) {
			newDayReminder.important = true;
		} 
		
		// Figure out where to put the new item in sorted list
		var treeitems = treeChildren.childNodes;
		var added = false;
		for (var i=0; i < treeitems.length; i++) {
			var item = treeitems[i];
			var row = item.childNodes[0];
			var dateCell = row.childNodes[0];
			var dateLabel = dateCell.getAttribute("label");
			var dateLabelDiv = dateLabel.split(" ");
			var month = reminderFox_getMonthAsInteger(dateLabelDiv[0]);
			var date = dateLabelDiv[1];
			
			if (month > parseInt(valueArray[0]) || (month == parseInt(valueArray[0]) && parseInt(dayVal) <= date ) ) {	
				if (month == parseInt(valueArray[0]) && date == parseInt(dayVal)) {
					if ( !isImportant ) {
						// if this reminder is not marked as important, add to the 
						// end of the list of today's reminders
						continue;
					}
				}
				treeChildren.insertBefore(newItem, item);
				added = true;
				
				// highlight the newly added reminder in the list
				var treeSelection = document.getElementById("reminderTree").view.selection;	
				if ( selectCalendarSync ) {
						treeSelection.select(i);
				}
				// make sure that newly selected row is shown in scroll pane
				var value = document.getElementById("treeparent").value;
				var tree = document.getElementById("reminderTree");					
				var boxobject = tree.boxObject;
				boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
				
				var first = boxobject.getFirstVisibleRow();
				var last = boxobject.getLastVisibleRow();
				if ( i < first || i > last ) {	
					boxobject.scrollToRow(treeitems.length);
					boxobject.scrollToRow(treeitems.length - 9);
				}

							
				var previousReminderOffset = 3;
				if ( i < previousReminderOffset ) { 
					boxobject.ensureRowIsVisible(i);
				}
				else {
					boxobject.ensureRowIsVisible(i  - previousReminderOffset);
				}
					
				break;
			}
		}
		
		if (!added){
			treeChildren.appendChild(newItem);
			// highlight the newly added reminder in the list
			var treeSelection = document.getElementById("reminderTree").view.selection;	
			if ( selectCalendarSync ) {
				treeSelection.select(treeitems.length);
			}
			// make sure that newly selected row is shown in scroll pane
			var value = document.getElementById("treeparent").value;
			var tree = document.getElementById("reminderTree");					
			var boxobject = tree.boxObject;
			boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
			
			var first = boxobject.getFirstVisibleRow();
			var last = boxobject.getLastVisibleRow();
			if ( i < first || i > last ) {	
				boxobject.scrollToRow(treeitems.length);			
				boxobject.scrollToRow(treeitems.length - 9);
			}
			
			var previousReminderOffset = 3;
			if ( i < previousReminderOffset ) { 
				boxobject.ensureRowIsVisible(i);
			}
			else {
				boxobject.ensureRowIsVisible(i  - previousReminderOffset);
			}
			
		}
	}
}


function createTodoListItem(valueArray, sort) {
	var treeChildren = document.getElementById("todoTreechildren");
	var newItem = document.createElement("treeitem");
	var newRow = document.createElement("treerow");
	var newDescLabel = document.createElement("treecell");
	
	newItem.appendChild(newRow);
	newRow.appendChild(newDescLabel);
	var isImportant = false;
	var showInTooltip = false; 
	
	newDescLabel.setAttribute("label", valueArray[0]);
		
	var newCompletedLabel = document.createElement("treecell");
	newRow.appendChild(newCompletedLabel);
	
	// check for any options that are specified, and mark them in the columns 
	if ( valueArray.length >= 2 ) {
        for ( j = 1; j < valueArray.length; j++ ) {
            var val = valueArray[j];
			 if ( val == REMINDER_IMPORTANT_FLAG ) {
                // mark the date and description as important (red-highlighted)
                newDescLabel.setAttribute("properties", "important");
                isImportant = true;
            }
            else if ( val.indexOf( REMINDER_TO_BE_COMPLETED_DELIMITER ) == 0 ) {
                newCompletedLabel.setAttribute("label", SHOW_IN_TOOLTIP_LABEL);	
                showInTooltip = true;		
            }
        }
    }
    
	var treeitems = treeChildren.childNodes;
	
	if( !sort || treeitems.length == 0 ) {
        treeChildren.appendChild(newItem);

        /*  
		// highlight the newly added reminder in the list 	     	
    	var treeSelection = document.getElementById("todoTree").view.selection;	
		treeSelection.select(treeitems.length - 1);
		
		var tree = document.getElementById("todoTree");					
		var boxobject = tree.boxObject;
		boxobject.ensureRowIsVisible(treeitems.length - 1);
		*/
	}
	
	
    // if current todo is not important, add after important items
    else if ( !isImportant) {
		var added = false;
		for (var i=0; i < treeitems.length && !added; i++) {
			var item = treeitems[i];
			var row = item.childNodes[0];
			var descCell = row.childNodes[0];
			var descLabel = descCell.getAttribute("label");		
			
			var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);

			var todoToRemove = descLabel;
			var indexOfReminderToRemove = todos.indexOf(todoToRemove);
			var endIndex = todos.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
			var totalnewReminder;
			var currentReminder;
		
			if ( endIndex != -1 ) {
				currentReminder = todos.substring( indexOfReminderToRemove, endIndex );
			}
			else {
				currentReminder = todos.substring( indexOfReminderToRemove, todos.length );
			}
			
			// search for important flag.  If one, remove it.  
			// if none, add it.  But add in front of one-time flag
			var indexImportant = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG );
			var currentImportant = indexImportant != -1;
		
			var indexTooltip = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER );
			var currentTooltip = indexTooltip != -1;
			
 			if ( !currentImportant ) {
	        	if ( showInTooltip ) {
	     			treeChildren.insertBefore(newItem, item);
					added = true;	        	
					// highlight the newly added reminder in the list
					var tree = document.getElementById("todoTree");	
					var treeSelection = tree.view.selection;	
					treeSelection.select(i);					
	
					var boxobject = tree.boxObject;
					boxobject.ensureRowIsVisible(i);
				}
	        	else {
	        		if ( !currentTooltip ) {
						treeChildren.insertBefore(newItem, item);
						added = true;
						// highlight the newly added reminder in the list
						var tree = document.getElementById("todoTree");	
						var treeSelection = tree.view.selection;	
						treeSelection.select(i);
						
						var boxobject = tree.boxObject;
						boxobject.ensureRowIsVisible(i);
	        		}
	        	}
	        
	        }
		}
		
    
    	if ( !added ) {
           	treeChildren.appendChild(newItem);
			// highlight the newly added reminder in the list 	 
	    	var tree = document.getElementById("todoTree");	
	    	var treeSelection = tree.view.selection;	
			treeSelection.select(treeitems.length);			
							
			var boxobject = tree.boxObject;
			boxobject.ensureRowIsVisible(treeitems.length );

			/*
			// make sure that newly selected row is shown in scroll pane
			var value = document.getElementById("todoTreeparent").value;
			var tree = document.getElementById("todoTree");					
			var boxobject = tree.boxObject;
			boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
			
			var first = boxobject.getFirstVisibleRow();
			var last = boxobject.getLastVisibleRow();
			if ( i < first || i > last ) {	
				boxobject.scrollToRow(treeitems.length);			
				boxobject.scrollToRow(treeitems.length - 9);
			}
			
			var previousReminderOffset = 3;
			if ( i < previousReminderOffset ) { 
				boxobject.ensureRowIsVisible(i);
			}
			else {
				boxobject.ensureRowIsVisible(i  - previousReminderOffset);
			}
	    	*/
       }
    }
    // new Todo is important - add to top of list
    else {        	
    	var item = treeitems[0];
    	treeChildren.insertBefore(newItem, item);
		added = true;
		
		// highlight the newly added reminder in the list
		var tree = document.getElementById("todoTree");	
		var treeSelection = tree.view.selection;	
		treeSelection.select(0);
						
		var boxobject = tree.boxObject;
		boxobject.ensureRowIsVisible(0);
		/*
		// make sure that newly selected row is shown in scroll pane
		var value = document.getElementById("todoTreeparent").value;
		var tree = document.getElementById("todoTree");					
		var boxobject = tree.boxObject;
		boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
		
		var first = boxobject.getFirstVisibleRow();
		var last = boxobject.getLastVisibleRow();
		if ( i < first || i > last ) {	
			boxobject.scrollToRow(treeitems.length);
			boxobject.scrollToRow(treeitems.length - 9);
		}

					
		var previousReminderOffset = 3;
		if ( i < previousReminderOffset ) { 
			boxobject.ensureRowIsVisible(i);
		}
		else {
			boxobject.ensureRowIsVisible(i  - previousReminderOffset);
		}
		*/
    }	
	
}
function createTodoListItemBefore(valueArray,  beforeTodo) {
	var treeChildren = document.getElementById("todoTreechildren");
	var newItem = document.createElement("treeitem");
	var newRow = document.createElement("treerow");
	var newDescLabel = document.createElement("treecell");
	
	newItem.appendChild(newRow);
	newRow.appendChild(newDescLabel);
	var isImportant = false;
	var showInTooltip = false; 
	
	newDescLabel.setAttribute("label", valueArray[0]);
		
	var newCompletedLabel = document.createElement("treecell");
	newRow.appendChild(newCompletedLabel);
	
	// check for any options that are specified, and mark them in the columns 
	if ( valueArray.length >= 2 ) {
        for ( j = 1; j < valueArray.length; j++ ) {
            var val = valueArray[j];
			 if ( val == REMINDER_IMPORTANT_FLAG ) {
                // mark the date and description as important (red-highlighted)
                newDescLabel.setAttribute("properties", "important");
                isImportant = true;
            }
            else if ( val.indexOf( REMINDER_TO_BE_COMPLETED_DELIMITER ) == 0 ) {
                newCompletedLabel.setAttribute("label", SHOW_IN_TOOLTIP_LABEL);	
                showInTooltip = true;		
            }
        }
    }
    
	var treeitems = treeChildren.childNodes;

	var added = false;
	for (var i=0; i < treeitems.length && !added; i++) {
		var item = treeitems[i];
		var row = item.childNodes[0];
		var descCell = row.childNodes[0];
		var descLabel = descCell.getAttribute("label");		
		
		if ( descLabel == beforeTodo ) {
				 treeChildren.insertBefore(newItem, item);
				added = true;	        	
				// highlight the newly added reminder in the list
				var tree = document.getElementById("todoTree");	
				var treeSelection = tree.view.selection;	
				treeSelection.select(i);					

				var boxobject = tree.boxObject;
				boxobject.ensureRowIsVisible(i);
		}
	}
		
    if ( !added ) {
        treeChildren.appendChild(newItem);
        // highlight the newly added reminder in the list 	 
        var tree = document.getElementById("todoTree");	
        var treeSelection = tree.view.selection;	
        treeSelection.select(treeitems.length );			
                        
        var boxobject = tree.boxObject;
        boxobject.ensureRowIsVisible(treeitems.length );
   }
 
	
}

/**
  * select (highlight) today's reminder or the next upcoming reminder
  */
function highlightClosestUpcomingReminder() {
	var currentDate = new Date();
	var currentMonth = parseInt(currentDate.getMonth());
	var currentDay = parseInt(currentDate.getDate() );

	var treeChildren = document.getElementById("treechildren");
	var treeitems = treeChildren.childNodes;
	for (var i=0; i < treeitems.length; i++) {
		var item = treeitems[i];
		var row = item.childNodes[0];
		var dateCell = row.childNodes[0];
		var dateLabel = dateCell.getAttribute("label");
		var dateLabelDiv = dateLabel.split(" ");
		var month = reminderFox_getMonthAsInteger(dateLabelDiv[0]);
		var date = parseInt(dateLabelDiv[1]);
		if ( month > currentMonth || (month == currentMonth && date >= currentDay ) ) {
			var treeSelection = document.getElementById("reminderTree").view.selection		
			treeSelection.select(i);
			
			// make sure that newly selected row is shown in scroll pane
			var value = document.getElementById("treeparent").value;
		  	var tree = document.getElementById("reminderTree");			
			var boxobject = tree.boxObject;
			boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
			
			// make sure it shows up at the top if possible
			var first = boxobject.getFirstVisibleRow();
			var last = boxobject.getLastVisibleRow();
			if ( i < first || i > last ) {				
				boxobject.scrollToRow(treeitems.length - 9);
			}
			
			var previousReminderOffset = 3;
			if ( i < previousReminderOffset ) { 
				boxobject.ensureRowIsVisible(i);
			}
			else {
				boxobject.ensureRowIsVisible(i  - previousReminderOffset);
			}
			break;
		}
	}
}

/**
  * select (highlight) today's reminder or the next upcoming reminder
  */
function highlightReminderForDate(currentMonth, currentDay) {
	var highlighted = false;
	var treeChildren = document.getElementById("treechildren");
	var treeitems = treeChildren.childNodes;
	for (var i=0; i < treeitems.length; i++) {
		var item = treeitems[i];
		var row = item.childNodes[0];
		var dateCell = row.childNodes[0];
		var dateLabel = dateCell.getAttribute("label");
		var dateLabelDiv = dateLabel.split(" ");
		var month = reminderFox_getMonthAsInteger(dateLabelDiv[0]);
		var date = parseInt(dateLabelDiv[1]);
		if ( month == currentMonth && date == currentDay ) {
			highlighted = true;
			var treeSelection = document.getElementById("reminderTree").view.selection		
			treeSelection.select(i);

			// make sure that newly selected row is shown in scroll pane
			var value = document.getElementById("treeparent").value;
		  	var tree = document.getElementById("reminderTree");			
			var boxobject = tree.boxObject;
			boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
			
			// make sure it shows up at the top if possible
			var first = boxobject.getFirstVisibleRow();
			var last = boxobject.getLastVisibleRow();
			
			if ( i <= first || i >= last ) {				
				boxobject.scrollToRow(treeitems.length - 9);
			}
			
			var previousReminderOffset = 3;
			if ( i < previousReminderOffset ) { 
				boxobject.ensureRowIsVisible(i);
			}
			else {
				boxobject.ensureRowIsVisible(i  - previousReminderOffset);
			}
			break;
		}
	}
	
	if ( !highlighted ) {
		selectNewReminderText();
	}
}

function highlightTodo() {
	var tree = document.getElementById("todoTree");			
	var treeSelection = tree.view.selection;	
	treeSelection.select(0);	
	treeSelection.clearSelection();
			
	var boxobject = tree.boxObject;
	boxobject.ensureRowIsVisible(0);	
}


function removeListItem(dateLabel, dateDesc) {
	var treeChildren = document.getElementById("treechildren");
	var treeitems = treeChildren.childNodes;
	for (var i=0; i < treeitems.length; i++) {
		var item = treeitems[i];
		var row = item.childNodes[0];
		var dateCell = row.childNodes[0];
		var dateCellLabel = dateCell.getAttribute("label");
		var descCell = row.childNodes[1];
		var descCellLabel = descCell.getAttribute("label");
		if (descCellLabel == dateDesc && dateCellLabel == dateLabel) {
			treeChildren.removeChild(item);
			break;
		}
	}
}
function removeTodoListItem(dateDesc) {
	var treeChildren = document.getElementById("todoTreechildren");
	var treeitems = treeChildren.childNodes;

	for (var i=0; i < treeitems.length; i++) {
		var item = treeitems[i];
		var row = item.childNodes[0];
			
		var descCell = row.childNodes[0];
		var descCellLabel = descCell.getAttribute("label");
		if (descCellLabel == dateDesc ) {
			treeChildren.removeChild(item);
			break;
		}
	}	
}


// saves and closes the 'edit reminders' dialog
function revert() {
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	// there were changes.  Prompt user to revert/cancel
	// get a reference to the prompt service component.
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
	                    .getService(Components.interfaces.nsIPromptService);

	var flags=promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_0 +
   	promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_1;     
	var msg =  "Are you sure you want to revert?  All of the changes made during this session will be lost.";
	var buttonPressed = promptService.confirmEx(window,"Revert changes?",
	  msg,
	  flags, "Revert", "Cancel", null, null, {});		  
	  
	// revert pressed
	if ( buttonPressed == 0 ) {		
		var revertReminders = false;
		var revertTodos = false;
		if ( origReminders != reminders ) {
			revertReminders = true;
		}
		if ( origTodos != todos ) {
			revertTodos = true;
		}
		removeAllListItems(revertReminders, revertTodos);
		
		reminderFox_prefs.setCharPref(REMINDER_FOX_REMINDERS, origReminders);
		reminderFox_prefs.setCharPref(REMINDER_FOX_TODOS, origTodos);	
		
		fillList(revertReminders, revertTodos);
		refreshCalendar();
					
		// grey out the rever buttons
		var revertButton= document.getElementById("revertbutton");
		revertButton.setAttribute( "disabled", true );
		
		var revertButtonTodos= document.getElementById("revertbuttonTodos");
		revertButtonTodos.setAttribute( "disabled", true );
	}

		
	
}

function removeAllListItems(removeReminders, removeTodos) {
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);

	if ( origReminders != reminders ) {
		var reminderTree = document.getElementById("reminderTree");
		var treeChildren = document.getElementById("treechildren");
		reminderTree.removeChild(treeChildren);
		
		var newchildren = document.createElement("treechildren");
		newchildren.setAttribute("id", "treechildren");
		newchildren.setAttribute("ondblclick", "if (event.button == 0) modifyReminderDescription()");
		reminderTree.appendChild(newchildren);
	}
	
	if ( origTodos != todos  ) {
		var todoTreeChildren = document.getElementById("todoTreechildren");
		var treeitems = todoTreeChildren.childNodes;
		for (var i=treeitems.length-1; i >= 0; i--) {
			var item = treeitems[i];
			todoTreeChildren.removeChild(item);
		}		
	}
}


// saves and closes the 'edit reminders' dialog
function saveSettings() {
	var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
	var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);	
	var windowEnumerator = windowManagerInterface.getEnumerator("navigator:browser");

	// update all of the browsers
	while (windowEnumerator.hasMoreElements()) {
		var currentWindow = windowEnumerator.getNext();		
		currentWindow.reminderFox_loadReminderFox(true);			
	}
	
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	origReminders = reminders;
	origTodos = todos;

	window.close();
}

// Adds the new reminder from the edit reminder dialog
function addReminder() {
	var monthName = document.getElementById("monthlist");
	var dayname = document.getElementById("daylist");
	var desc = document.getElementById("newReminderText");
	var once = document.getElementById("occurrence");
	var important = document.getElementById("important");
	//var complete = document.getElementById("remindUntilComplete");
	
	if (desc.value == "")
		desc.value = "No Description";

    // make sure there's no special (separator) characters being 
    // used; if so, replace them
    desc.value = desc.value.replace( /~/g, "-" );
    desc.value = desc.value.replace( /`/g, "'" );
	
	var prefMonth = reminderFox_getMonthAsInteger(monthName.label);

	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	var newreminder = prefMonth + REMINDER_PROPERTY_DELIMITER + dayname.label +REMINDER_PROPERTY_DELIMITER + desc.value;
	
	
	// if this is marked as Important, denote that in the preferences
	if ( important.checked ) {
		newreminder = newreminder + REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG;
	}
	
	// if this is a one-time-only reminder, denote that in the preferences
	if ( once.checked && prefMonth != REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
		newreminder = newreminder + REMINDER_PROPERTY_DELIMITER + REMINDER_FOX_OCCURRENCE_ONCE;
	}

	// add reminder in sorted order...
	reminders = reminderFox_sortNewReminder( reminders, newreminder, prefMonth, parseInt( dayname.label ) );

	reminderFox_prefs.setCharPref(REMINDER_FOX_REMINDERS, reminders);
	createListItem(newreminder.split(REMINDER_PROPERTY_DELIMITER), false);
	desc.value = null;
			
	var newMonth = prefMonth;
	if ( prefMonth == REMINDER_FOX_MONTHLY_WILDCARD_CHAR	) {
		newMonth = 11;  // set to December
	}	
	var year = oeDatePicker.gSelectedDate.getFullYear();
	if ( year == null ) {
		year = new Date().getFullYear();
	}
	var newDate = new Date( year, newMonth, dayname.label );	
	oeDatePicker.onpopupshowing(newDate);
	
	modifiedReminders();
	
}

function modifiedReminders() {
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);

	if ( origReminders != reminders || origTodos != todos  ) {
		var revertButton= document.getElementById("revertbutton");
		revertButton.setAttribute( "disabled", false );
		
		var revertButtonTodos= document.getElementById("revertbuttonTodos");
		revertButtonTodos.setAttribute( "disabled", false );
	}
}

function addTodo() {
	var desc = document.getElementById("newTodoText");
	var important = document.getElementById("importantTodo");
	var showInTooltip = document.getElementById("showInTooltip");
	
	if (desc.value == "")
		desc.value = "No Description";

    // make sure there's no special (separator) characters being 
    // used; if so, replace them
    desc.value = desc.value.replace( /~/g, "-" );
    desc.value = desc.value.replace( /`/g, "'" );
	
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	var newtodo = desc.value;
	
	
	// if this is marked as Important, denote that in the preferences
	if ( important.checked ) {
		newtodo = newtodo + REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG;
	}
	
	// if this is marked as Remind Until Completed, denote that in the preferences
	if ( showInTooltip.checked ) {
		newtodo = newtodo + REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER ;		
	}
	
	// add todo in sorted order...
	todos = reminderFox_sortNewTodo( todos, newtodo );

	reminderFox_prefs.setCharPref(REMINDER_FOX_TODOS, todos);
	createTodoListItem(newtodo.split(REMINDER_PROPERTY_DELIMITER), true);
	desc.value = null;

	modifiedReminders();
}


function removeReminder() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];

	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
	var monthlyLabel = selectedTreeItem.childNodes[0].childNodes[3].getAttribute("label");
	var isMonthly = monthlyLabel == MONTHLY_LABEL;
	
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	
	var dateLabelArray = dateLabel.split(" ");
	var monthAsInteger = reminderFox_getMonthAsInteger(dateLabelArray[0]);
	if ( isMonthly ) {
	    monthAsInteger = REMINDER_FOX_MONTHLY_WILDCARD_CHAR;
	}
	
	var reminderToRemove = monthAsInteger+ REMINDER_PROPERTY_DELIMITER + dateLabelArray[1] + REMINDER_PROPERTY_DELIMITER + descLabel;
	var indexOfReminderToRemove = reminders.indexOf(reminderToRemove);
	
	var leftSideOfReminders; 
	var rightSideOfReminders;
	var endIndex = reminders.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	if ( endIndex != -1 ) {
		leftSideOfReminders = reminders.substring(0, indexOfReminderToRemove );
		if ( indexOfReminderToRemove > 0 ) {	
			leftSideOfReminders = leftSideOfReminders.substring(0, leftSideOfReminders.length -1 );  // remove REMINDER_SEPARATOR
		}
		rightSideOfReminders = reminders.substring(endIndex,  reminders.length);
	}
	else {
		leftSideOfReminders = reminders.substring(0, indexOfReminderToRemove );
		rightSideOfReminders = "";
	}
	
	// trim separators from beginning or end
	var finalReminders = leftSideOfReminders + rightSideOfReminders
	if ( finalReminders.indexOf( REMINDER_SEPARATOR ) == 0 ) {
		finalReminders = finalReminders.substring( 1, finalReminders.length );
	}	
	if ( finalReminders.lastIndexOf( REMINDER_SEPARATOR ) == finalReminders.length - 1 ) {
		finalReminders = finalReminders.substring( 0, finalReminders.length - 1);
	}

	reminderFox_prefs.setCharPref(REMINDER_FOX_REMINDERS, finalReminders);
	if ( isMonthly ) {
		removeMonthlyListItems(dateLabelArray[1], descLabel);
	}
	else {
		removeListItem(dateLabel, descLabel);
	}
}


function removeTodo() {
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var treeChildren = document.getElementById("todoTreechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];

	var descLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	
	var reminderToRemove = descLabel;

	var indexOfReminderToRemove = todos.indexOf(reminderToRemove);
	
	var leftSideOfReminders; 
	var rightSideOfReminders;
	var endIndex = todos.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	if ( endIndex != -1 ) {
		leftSideOfReminders = todos.substring(0, indexOfReminderToRemove );
		if ( indexOfReminderToRemove > 0 ) {	
			leftSideOfReminders = leftSideOfReminders.substring(0, leftSideOfReminders.length -1 );  // remove REMINDER_SEPARATOR
		}
		rightSideOfReminders = todos.substring(endIndex,  todos.length);
	}
	else {
		leftSideOfReminders = todos.substring(0, indexOfReminderToRemove );
		rightSideOfReminders = "";
	}

	// trim separators from beginning or end
	var finalReminders = leftSideOfReminders + rightSideOfReminders
	if ( finalReminders.indexOf( REMINDER_SEPARATOR ) == 0 ) {
		finalReminders = finalReminders.substring( 1, finalReminders.length );
	}	
	if ( finalReminders.lastIndexOf( REMINDER_SEPARATOR ) == finalReminders.length - 1 ) {
		finalReminders = finalReminders.substring( 0, finalReminders.length - 1);
	}

	reminderFox_prefs.setCharPref(REMINDER_FOX_TODOS, finalReminders);
	removeTodoListItem(descLabel);
}

function removeMonthlyListItems(dateLabel, dateDesc) {
	for ( var i = 0; i < 12; i++ ) {
		var month = reminderFox_getMonthAsText( i );
		var date = month + " " + dateLabel;
		removeListItem( date, dateDesc );
	}
}

function activateContext(event) {
	var tree = document.getElementById("reminderTree");
	var treeContextMenu = document.getElementById("treechildren-contextmenu");

	var xCoord = event.clientX;
	var yCoord = event.clientY;
	document.popupNode = tree;

	
	// modify context menu depending on Important reminder status
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	if ( selectedTreeItemIndex == -1 ) {
		return;
	}
	treeContextMenu.showPopup(tree, xCoord, yCoord, "bottomleft", "topleft");
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
	var monthlyLabel = selectedTreeItem.childNodes[0].childNodes[3].getAttribute("label");
	var isMonthly = monthlyLabel == MONTHLY_LABEL;
	
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);

	var dateLabelArray = dateLabel.split(" ");
	var monthAsInteger = reminderFox_getMonthAsInteger(dateLabelArray[0]);
	if ( isMonthly ) {
	    monthAsInteger = REMINDER_FOX_MONTHLY_WILDCARD_CHAR;
	}
	
	var reminderToRemove = monthAsInteger+ REMINDER_PROPERTY_DELIMITER + dateLabelArray[1] + REMINDER_PROPERTY_DELIMITER + descLabel;
	var indexOfReminderToRemove = reminders.indexOf(reminderToRemove);
	var endIndex = reminders.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	
	var totalnewReminder;
	var currentReminder;
	if ( endIndex != -1 ) {
		currentReminder = reminders.substring( indexOfReminderToRemove, endIndex );
	}
	else {
		currentReminder = reminders.substring( indexOfReminderToRemove, reminders.length );
	}
	
	var contextMenuImportant = document.getElementById("treechildren-contextmenu-important");	
	var indexImportant = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG );
	if ( indexImportant == -1 ) {
		contextMenuImportant.setAttribute("checked", "false");	
	}
	else {
		contextMenuImportant.setAttribute("checked", "true");	
	}
	
	var contextMenuRemindComplete = document.getElementById("treechildren-contextmenu-remindUntilComplete");	
	var contextMenuComplete = document.getElementById("treechildren-contextmenu-markAsComplete");	
	
	// search for completed
	var indexCompleted = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER );
	if ( indexCompleted != -1 ) {
		var endIndex = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER, indexCompleted + 1  );
		if ( endIndex == -1 ) {
			endIndex = currentReminder.length - 1;
		}
		else {
			endIndex = endIndex - 1;
		}
		
		// if a length of 1 only, then it's marked as complete. 
		if ( indexCompleted + 1 == endIndex )  {
			contextMenuComplete.setAttribute("checked", "true");	    
		}
		// if there's a date too, then it's remind until completed
		else {
			contextMenuRemindComplete.setAttribute("checked", "true");	 
		}
	}
	else {
		contextMenuRemindComplete.setAttribute("checked", "false");	 
		contextMenuComplete.setAttribute("checked", "false");	 
	}
}

function activateTodoContext(event) {
	var tree = document.getElementById("todoTree");
	var treeContextMenu = document.getElementById("todo-treechildren-contextmenu");

	var xCoord = event.clientX;
	var yCoord = event.clientY;
	document.popupNode = tree;
	
	// modify context menu depending on Important reminder status
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var treeChildren = document.getElementById("todoTreechildren");
	if ( treeChildren == null || selectedTreeItemIndex == -1 || treeChildren.childNodes.length == 0 ) {
		return;
	}
	treeContextMenu.showPopup(tree, xCoord, yCoord, "bottomleft", "topleft");
	
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var descLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
		
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	
	var reminderToRemove = descLabel;
	var indexOfReminderToRemove = todos.indexOf(reminderToRemove);
	var endIndex = todos.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	
	var totalnewReminder;
	var currentReminder;
	if ( endIndex != -1 ) {
		currentReminder = todos.substring( indexOfReminderToRemove, endIndex );
	}
	else {
		currentReminder = todos.substring( indexOfReminderToRemove, todos.length );
	}
	
	var contextMenuImportant = document.getElementById("treechildren-contextmenu-important2");	
	
	var indexImportant = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG );
	if ( indexImportant == -1 ) {
		contextMenuImportant.setAttribute("checked", "false");	
	}
	else {
		contextMenuImportant.setAttribute("checked", "true");	
	}
	
	// search for completed
	var contextMenuShowInTooltip = document.getElementById("treechildren-contextmenu-showInTooltip");	
	
	var indexCompleted = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER );
	if ( indexCompleted == -1 ) {
		contextMenuShowInTooltip.setAttribute("checked", "false");	
	}
	else {
		contextMenuShowInTooltip.setAttribute("checked", "true");	
	}
	
	// disable moveUp/moveDown if that option isn't available
	var moveUpMenu = document.getElementById("treechildren-contextmenu-moveUp");	
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	if ( selectedTreeItemIndex == 0 ) {
		moveUpMenu.setAttribute( "disabled", true );
	}
	else {
		moveUpMenu.setAttribute( "disabled", false );
	}
	
	var moveDownMenu = document.getElementById("treechildren-contextmenu-moveDown");	
	var treeitems = treeChildren.childNodes;
	
	if ( selectedTreeItemIndex == (treeitems.length - 1) ) {
		moveDownMenu.setAttribute( "disabled", true );
	}
	else {
		moveDownMenu.setAttribute( "disabled", false );
	}	
}


function processKeyPressedOnReminderTree(event) {
	if (event.keyCode == 46)
		userDeleteReminder();
}
function processKeyPressedOnTodoTree(event) {
	if (event.keyCode == 46)
		userDeleteTodo();
}


function modifyTodoDescription()
{
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var treeChildren = document.getElementById("todoTreechildren");
	if ( selectedTreeItemIndex == -1 ) {
		return;
	}
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	if ( selectedTreeItem == null ) {
		return;
	}
	var descLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");

	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	var result = { value: descLabel };
	var dummy = { value: 0 };
	if (gPromptService.prompt(window,
                            "Description",
                            "Modify description for ToDo:",
                            result,
                            null,
                            dummy)) {
    
  	// to update, first remove old reminder     
    var updatedReminder = getUpdatedTodo( descLabel, result.value );
    removeTodo();

	// add new reminder
    var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
 
	todos = reminderFox_sortNewTodo( todos, updatedReminder );
	reminderFox_prefs.setCharPref(REMINDER_FOX_TODOS, todos);
	createTodoListItem(updatedReminder.split(REMINDER_PROPERTY_DELIMITER), true);

	modifiedReminders();
  }
}

function getUpdatedTodo( descLabel, newDesc ) {
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);	
	var todoToRemove = descLabel;

	var indexOfReminderToRemove = todos.indexOf(todoToRemove);
	var endIndex = todos.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	var totalnewReminder;
	var newReminderStr = newDesc;
	var newReminderEnd;
	if ( endIndex != -1 ) {		
		newReminderEnd = todos.substring( indexOfReminderToRemove + todoToRemove.length, endIndex );		
	}
	else {		
		newReminderEnd = todos.substring( indexOfReminderToRemove + todoToRemove.length, todos.length );		
	}
	totalnewReminder = newReminderStr + newReminderEnd;
	return totalnewReminder;
}


function modifyReminderDescription() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	 
	if ( selectedTreeItemIndex == -1 ) {
		return;
	}
	
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	if (  selectedTreeItem == null ) {
		return;
	}
	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
	var monthlyLabel = selectedTreeItem.childNodes[0].childNodes[3].getAttribute("label");
	var isMonthly = monthlyLabel == MONTHLY_LABEL;

	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	var result = { value: descLabel };
	var dummy = { value: 0 };
	if (gPromptService.prompt(window,
                            "Description",
                            "Modify description for " + dateLabel + " reminder:",
                            result,
                            null,
                            dummy)) {
    
	  	// to update, first remove old reminder     
	    var updatedReminder = getUpdatedReminder( dateLabel, descLabel, isMonthly, result.value );
	    removeReminder();
	
		// add new reminder
	    var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	    var dateLabelArray = dateLabel.split(" ");
		var monthAsInteger = reminderFox_getMonthAsInteger(dateLabelArray[0]);
		if ( isMonthly ) {
		    monthAsInteger = REMINDER_FOX_MONTHLY_WILDCARD_CHAR;
		}
		var dayAsInteger = parseInt( dateLabelArray[1] );
	
		reminders = reminderFox_sortNewReminder( reminders, updatedReminder, monthAsInteger, dayAsInteger );
		reminderFox_prefs.setCharPref(REMINDER_FOX_REMINDERS, reminders);
		createListItem(updatedReminder.split(REMINDER_PROPERTY_DELIMITER), false);

		refreshCalendar();
		
		modifiedReminders();
  	}
}

function userDeleteReminder() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];

	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
	
	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	if (gPromptService.confirm(window,
                            "Delete Reminder",
                            "Press OK to delete the reminder\n  " + dateLabel + ": " + descLabel   )) {
     	removeReminder();
     	
		refreshCalendar();
		
		modifiedReminders();
     }
}

function userMoveReminderToTodo() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];

	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
	
	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	if (gPromptService.confirm(window,
                            "Move to ToDo",
                            "Press OK to move the following reminder to the ToDo list:\n  " + descLabel   )) {
       	var monthlyLabel = selectedTreeItem.childNodes[0].childNodes[3].getAttribute("label");
		var isMonthly = monthlyLabel == MONTHLY_LABEL;
	
		var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);

		var dateLabelArray = dateLabel.split(" ");
		var monthAsInteger = reminderFox_getMonthAsInteger(dateLabelArray[0]);
		if ( isMonthly ) {
		    monthAsInteger = REMINDER_FOX_MONTHLY_WILDCARD_CHAR;
		}
	
		var reminderToRemove = monthAsInteger+ REMINDER_PROPERTY_DELIMITER + dateLabelArray[1] + REMINDER_PROPERTY_DELIMITER + descLabel;
		var indexOfReminderToRemove = reminders.indexOf(reminderToRemove);
		var endIndex = reminders.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
		var totalnewReminder;
		var currentReminder;
		if ( endIndex != -1 ) {
			currentReminder = reminders.substring( indexOfReminderToRemove, endIndex );
		}
		else {
			currentReminder = reminders.substring( indexOfReminderToRemove, reminders.length );
		}
		var indexImportant = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG );
		
     	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
		var newtodo = descLabel;	
	
		// if this is marked as Important, denote that in the preferences
		if ( indexImportant != -1 ) {
			newtodo = newtodo + REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG;
		}
	
		// add reminder in sorted order...
		todos = reminderFox_sortNewTodo( todos, newtodo );
		//todos = todos + REMINDER_SEPARATOR + newtodo;
		reminderFox_prefs.setCharPref(REMINDER_FOX_TODOS, todos);
		createTodoListItem(newtodo.split(REMINDER_PROPERTY_DELIMITER), true);
	     	
	    removeReminder();
	    
	    refreshCalendar();
	    
    	modifiedReminders();	    	
  	}
}

function userDeleteTodo() {
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var treeChildren = document.getElementById("todoTreechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];

	var descLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	
	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	if (gPromptService.confirm(window,
                            "Delete ToDo",
                            "Press OK to delete the ToDo:\n  " + descLabel   )) {
     	removeTodo();
     	
  		modifiedReminders();
     }
}

function getUpdatedReminder( dateLabel, descLabel, isMonthly, newDesc ) {
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	var dateLabelArray = dateLabel.split(" ");
	var monthAsInteger = reminderFox_getMonthAsInteger(dateLabelArray[0]);
	if ( isMonthly ) {
	    monthAsInteger = REMINDER_FOX_MONTHLY_WILDCARD_CHAR;
	}
	
	var reminderToRemove = monthAsInteger+ REMINDER_PROPERTY_DELIMITER + dateLabelArray[1] + REMINDER_PROPERTY_DELIMITER + descLabel;

	var indexOfReminderToRemove = reminders.indexOf(reminderToRemove);
	var endIndex = reminders.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	var totalnewReminder;
	var newReminderStr = monthAsInteger+ REMINDER_PROPERTY_DELIMITER + dateLabelArray[1] + REMINDER_PROPERTY_DELIMITER + newDesc;
	var newReminderEnd;
	if ( endIndex != -1 ) {		
		newReminderEnd = reminders.substring( indexOfReminderToRemove + reminderToRemove.length, endIndex );		
	}
	else {		
		newReminderEnd = reminders.substring( indexOfReminderToRemove + reminderToRemove.length, reminders.length );		
	}
	totalnewReminder = newReminderStr + newReminderEnd;
	return totalnewReminder;
}



function moveTodoUp() {
	// only handle events when on the ToDos tab.
	var todosTab = document.getElementById("todosTab");
	if ( !todosTab.selected ) {
		return;
	}
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	if ( selectedTreeItemIndex == 0 ) {
		return;
	} 
	
	var treeChildren = document.getElementById("todoTreechildren");
	var previousTreeItem = treeChildren.childNodes[selectedTreeItemIndex - 1];
	var previousDescLabel = previousTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	
	
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var descLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);

	// toggle important flag now
	
	var todoToRemove = descLabel;
	var indexOfReminderToRemove = todos.indexOf(todoToRemove);
	var endIndex = todos.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	var totalnewReminder;
	var currentReminder;

	if ( endIndex != -1 ) {
		currentReminder = todos.substring( indexOfReminderToRemove, endIndex );
	}
	else {
		currentReminder = todos.substring( indexOfReminderToRemove, todos.length );
	}
	
	
	// now have new reminder str; delete old one and add this new one
	removeTodo();

	// add new reminder
    var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);

	todos = reminderFox_sortNewTodoBefore( todos, currentReminder, previousDescLabel  );
	reminderFox_prefs.setCharPref(REMINDER_FOX_TODOS, todos);

	createTodoListItemBefore(currentReminder.split(REMINDER_PROPERTY_DELIMITER),  previousDescLabel);
	
	modifiedReminders();
	
}



function moveTodoDown() {
	// only handle events when on the ToDos tab.
	var todosTab = document.getElementById("todosTab");
	if ( !todosTab.selected ) {
		return;
	}
	
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;

	
	var treeChildren = document.getElementById("todoTreechildren");
	var treeitems = treeChildren.childNodes;
	var previousDescLabel = null;
	
	// if the last item is already selected, just exit
	if ( selectedTreeItemIndex == (treeitems.length - 1) ) {
		return;
	}

	if ( (selectedTreeItemIndex + 2) < treeitems.length ) {
		var twoAhead = treeChildren.childNodes[selectedTreeItemIndex +2];
		 previousDescLabel = twoAhead.childNodes[0].childNodes[0].getAttribute("label");
	}
	
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var descLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	
	var todoToRemove = descLabel;
	var indexOfReminderToRemove = todos.indexOf(todoToRemove);
	var endIndex = todos.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	var totalnewReminder;
	var currentReminder;

	if ( endIndex != -1 ) {
		currentReminder = todos.substring( indexOfReminderToRemove, endIndex );
	}
	else {
		currentReminder = todos.substring( indexOfReminderToRemove, todos.length );
	}
	
	
	// now have new reminder str; delete old one and add this new one
	removeTodo();

	// add new reminder
    var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	todos = reminderFox_sortNewTodoBefore( todos, currentReminder, previousDescLabel  );
	reminderFox_prefs.setCharPref(REMINDER_FOX_TODOS, todos);
	createTodoListItemBefore(currentReminder.split(REMINDER_PROPERTY_DELIMITER),  previousDescLabel);
	
	modifiedReminders();	
}


function toggleTodoImportantFlag() {
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var treeChildren = document.getElementById("todoTreechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var descLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);

	// toggle important flag now
	
	var todoToRemove = descLabel;
	var indexOfReminderToRemove = todos.indexOf(todoToRemove);
	var endIndex = todos.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	var totalnewReminder;
	var currentReminder;

	if ( endIndex != -1 ) {
		currentReminder = todos.substring( indexOfReminderToRemove, endIndex );
	}
	else {
		currentReminder = todos.substring( indexOfReminderToRemove, todos.length );
	}
	
	// search for important flag.  If one, remove it.  
	// if none, add it.  But add in front of one-time flag
	var indexImportant = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG );
	if ( indexImportant != -1 ) {
		// this is marked as important...  Remove it.  
		currentReminder = currentReminder.substring( 0, indexImportant ) + currentReminder.substring( indexImportant + 2, currentReminder.length );
	}
	else {
		// not marked as important - add it as important before once delimiter
		var index = currentReminder.indexOf( descLabel );
		var newReminder  = currentReminder.substring( 0, index + descLabel.length) + 
			REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG +
			currentReminder.substring( index + descLabel.length, currentReminder.length );
		currentReminder = newReminder;								
	}
	
	// now have new reminder str; delete old one and add this new one
	removeTodo();

	// add new reminder
    var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	
	todos = reminderFox_sortNewTodo( todos, currentReminder );
	reminderFox_prefs.setCharPref(REMINDER_FOX_TODOS, todos);
	createTodoListItem(currentReminder.split(REMINDER_PROPERTY_DELIMITER), true);
	
	modifiedReminders();	
}


function toggleImportantFlag() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
	var monthlyLabel = selectedTreeItem.childNodes[0].childNodes[3].getAttribute("label");
	var isMonthly = monthlyLabel == MONTHLY_LABEL;
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);

	// toggle important flag now
	var dateLabelArray = dateLabel.split(" ");
	var monthAsInteger = reminderFox_getMonthAsInteger(dateLabelArray[0]);
	if ( isMonthly ) {
	    monthAsInteger = REMINDER_FOX_MONTHLY_WILDCARD_CHAR;
	}

	var reminderToRemove = monthAsInteger+ REMINDER_PROPERTY_DELIMITER + dateLabelArray[1] + REMINDER_PROPERTY_DELIMITER + descLabel;
	var indexOfReminderToRemove = reminders.indexOf(reminderToRemove);
	var endIndex = reminders.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	var totalnewReminder;
	var currentReminder;

	if ( endIndex != -1 ) {
		currentReminder = reminders.substring( indexOfReminderToRemove, endIndex );
	}
	else {
		currentReminder = reminders.substring( indexOfReminderToRemove, reminders.length );
	}

	// search for important flag.  If one, remove it.  
	// if none, add it.  But add in front of one-time flag
	var indexImportant = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG );
	if ( indexImportant != -1 ) {
		// this is marked as important...  Remove it.  
		currentReminder = currentReminder.substring( 0, indexImportant ) + currentReminder.substring( indexImportant + 2, currentReminder.length );
	}
	else {
		// not marked as important - add it as important before once delimiter
		var index = currentReminder.indexOf( descLabel );
		var newReminder  = currentReminder.substring( 0, index + descLabel.length) + 
			REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG +
			currentReminder.substring( index + descLabel.length, currentReminder.length );
		currentReminder = newReminder;								
	}

	// now have new reminder str; delete old one and add this new one
	removeReminder();
	
	// add new reminder
    var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	var dayAsInteger = parseInt( dateLabelArray[1] );

	reminders = reminderFox_sortNewReminder( reminders, currentReminder, monthAsInteger, dayAsInteger );
	reminderFox_prefs.setCharPref(REMINDER_FOX_REMINDERS, reminders);

	createListItem(currentReminder.split(REMINDER_PROPERTY_DELIMITER), false);

	refreshCalendar();
	
	modifiedReminders();
}

function refreshCalendar() {
	calendarReminderArray = null;	
	oeDatePicker.redrawDays();
}


function toggleTodoShowInTooltip() {
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var treeChildren = document.getElementById("todoTreechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var descLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	
	var todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	
	var reminderToRemove = descLabel;
	var indexOfReminderToRemove = todos.indexOf(reminderToRemove);
	var endIndex = todos.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	var totalnewReminder;
	var currentTodo;
	

	if ( endIndex != -1 ) {
		currentTodo = todos.substring( indexOfReminderToRemove, endIndex );
	}
	else {
		currentTodo = todos.substring( indexOfReminderToRemove, todos.length );
	}
	
	// search for completed flag.  
	var indexCompleted = currentTodo.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER );	
	var todoToAdd;
	if ( indexCompleted != -1 ) {
		// we want to mark this as Completed....   
        // Delete current entry and add back to the old date
		todoToAdd = currentTodo.substring( 0, indexCompleted ) + 
			currentTodo.substring( indexCompleted + 2, currentTodo.length );
	}
	else {
		// mark as Remind Until Completed

		todoToAdd = currentTodo + REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER;
	}
	
	// now have new todo str; delete old one and add this new one
	removeTodo();
	todos = reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	
	// add todo in sorted order...
	todos = reminderFox_sortNewTodo( todos, todoToAdd );
	reminderFox_prefs.setCharPref(REMINDER_FOX_TODOS, todos);
	createTodoListItem(todoToAdd.split(REMINDER_PROPERTY_DELIMITER), true);
	
	modifiedReminders();
}

function toggleRemindUntilCompleted() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
	var monthlyLabel = selectedTreeItem.childNodes[0].childNodes[3].getAttribute("label");
	var isMonthly = monthlyLabel == MONTHLY_LABEL;
	
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);

	var dateLabelArray = dateLabel.split(" ");
	var monthAsInteger = reminderFox_getMonthAsInteger(dateLabelArray[0]);
	if ( isMonthly ) {
	    monthAsInteger = REMINDER_FOX_MONTHLY_WILDCARD_CHAR;
	}
	
	var reminderToRemove = monthAsInteger+ REMINDER_PROPERTY_DELIMITER + dateLabelArray[1] + REMINDER_PROPERTY_DELIMITER + descLabel;
	var indexOfReminderToRemove = reminders.indexOf(reminderToRemove);
	var endIndex = reminders.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	var totalnewReminder;
	var currentReminder;
	

	if ( endIndex != -1 ) {
		currentReminder = reminders.substring( indexOfReminderToRemove, endIndex );
	}
	else {
		currentReminder = reminders.substring( indexOfReminderToRemove, reminders.length );
	}
	
	// search for completed flag.  
	var indexCompleted = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER );	
		
	if ( indexCompleted != -1 ) {
		// we want to mark this as Completed....   
        // Delete current entry and add back to the old date
			
		// keep track of old date
		var endIndex = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER, indexCompleted + 1 );
		if ( endIndex == -1 ) {
			endIndex = currentReminder.length - 1;
		}
		else {
			endIndex = endIndex - 1;
		}
		var dateSeparatorIndex = currentReminder.indexOf( "-", indexCompleted );
		var newIndex = indexCompleted + 2;
		var month = currentReminder.substring( newIndex, dateSeparatorIndex );
		newIndex = dateSeparatorIndex + 1;
		var day = currentReminder.substring( newIndex, endIndex + 1 );

		var newReminder = month + REMINDER_PROPERTY_DELIMITER + day + REMINDER_PROPERTY_DELIMITER;	
		var remArray = currentReminder.split(REMINDER_PROPERTY_DELIMITER);
		for ( j=2; j < remArray.length; j++ ) {
			if ( remArray[j].indexOf( REMINDER_TO_BE_COMPLETED_DELIMITER ) != 0  ) {				
				newReminder = newReminder + remArray[j]; 
				if ( j < remArray.length - 1 ) {
					newReminder = newReminder +	REMINDER_PROPERTY_DELIMITER;
				}			
			}
		}

		if ( newReminder.lastIndexOf( REMINDER_PROPERTY_DELIMITER ) == newReminder.length - 1 ) {
			newReminder = newReminder.substring( 0, newReminder.length - 1 );
		}
		currentReminder = newReminder;	
	}
	else {
		// mark as Remind Until Completed
		
		// keep track of old date
		var remArray = currentReminder.split(REMINDER_PROPERTY_DELIMITER);
		var completedStr = REMINDER_TO_BE_COMPLETED_DELIMITER + remArray[0] + "-" + remArray[1];
		var newReminder = "";
		var onceindex = currentReminder.lastIndexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_FOX_OCCURRENCE_ONCE );
		if ( onceindex == currentReminder.length - 2 ) {
			newReminder = currentReminder.substring(0, onceindex ) + REMINDER_PROPERTY_DELIMITER + completedStr + REMINDER_PROPERTY_DELIMITER + REMINDER_FOX_OCCURRENCE_ONCE;
		} 
		else {
			newReminder = currentReminder + REMINDER_PROPERTY_DELIMITER + completedStr;
		}
		currentReminder = newReminder;		
	}
	
	// now have new reminder str; delete old one and add this new one
	removeReminder();

	// add new reminder
    var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	var dayAsInteger = parseInt( dateLabelArray[1] );

	reminders = reminderFox_sortNewReminder( reminders, currentReminder, monthAsInteger, dayAsInteger );
	reminderFox_prefs.setCharPref(REMINDER_FOX_REMINDERS, reminders);
	createListItem(currentReminder.split(REMINDER_PROPERTY_DELIMITER), false);
	
	modifiedReminders();
}


function toggleMarkAsComplete() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
	var monthlyLabel = selectedTreeItem.childNodes[0].childNodes[3].getAttribute("label");
	var isMonthly = monthlyLabel == MONTHLY_LABEL;
	
	var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);

	var dateLabelArray = dateLabel.split(" ");
	var monthAsInteger = reminderFox_getMonthAsInteger(dateLabelArray[0]);
	if ( isMonthly ) {
	    monthAsInteger = REMINDER_FOX_MONTHLY_WILDCARD_CHAR;
	}
	
	var reminderToRemove = monthAsInteger+ REMINDER_PROPERTY_DELIMITER + dateLabelArray[1] + REMINDER_PROPERTY_DELIMITER + descLabel;
	var indexOfReminderToRemove = reminders.indexOf(reminderToRemove);
	var endIndex = reminders.indexOf( REMINDER_SEPARATOR, indexOfReminderToRemove + 1 );
	var totalnewReminder;
	var currentReminder;
	

	if ( endIndex != -1 ) {
		currentReminder = reminders.substring( indexOfReminderToRemove, endIndex );
	}
	else {
		currentReminder = reminders.substring( indexOfReminderToRemove, reminders.length );
	}
	
	// search for completed flag.  
	var indexCompleted = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER );	
		
	if ( indexCompleted == -1 ) {
		// we want to mark this as Completed....   
		currentReminder = currentReminder + REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER;
	}
	else {
		// mark as Remind Until Completed
		// check for Completed status
		var endIndex = currentReminder.indexOf( REMINDER_PROPERTY_DELIMITER, indexCompleted + 1  );
		if ( endIndex == -1 ) {
			endIndex = currentReminder.length - 1;
		}
		else {
			endIndex = endIndex - 1;
		}
		
		// if a length of 1 only, then it's marked as complete.  Skip it.
		if ( indexCompleted + 1 == endIndex )  {
			// remove the Complete tag
			currentReminder = currentReminder.substring( 0, indexCompleted ) +
				currentReminder.substring( indexCompleted + 2 );
			
		}
		else {
			// it's got a date in it.  Handle that scenario.
			toggleRemindUntilCompleted();
			return;
		}
	}
	
	// now have new reminder str; delete old one and add this new one
	removeReminder();

	// add new reminder
    var reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	var dayAsInteger = parseInt( dateLabelArray[1] );

	reminders = reminderFox_sortNewReminder( reminders, currentReminder, monthAsInteger, dayAsInteger );
	reminderFox_prefs.setCharPref(REMINDER_FOX_REMINDERS, reminders);
	createListItem(currentReminder.split(REMINDER_PROPERTY_DELIMITER), false);
	
	modifiedReminders();
}